@@ -40,25 +40,30 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-gTestfile = '13.4.4.17.js';
+gTestfile = '13.4.4.14.js';
 
-START("13.4.4.17 - XML inScopeNamespaces()");
-
-TEST(1, true, XML.prototype.hasOwnProperty("inScopeNamespaces"));
+START("13.4.4.14 - XML hasOwnProperty()");
 
+TEST(1, true, XML.prototype.hasOwnProperty("hasOwnProperty"));
+   
 x =
-<alpha xmlns:foo="http://foo/" xmlns:bar="http://bar/">
+<alpha attr1="value1">
     <bravo>one</bravo>
+    <charlie>
+        two
+        three
+        <echo>four</echo>
+    </charlie>
+    <delta />
 </alpha>;
 
-namespaces = x.bravo.inScopeNamespaces();
+// Returns true for elements/attributes
+TEST(2, true, x.hasOwnProperty("bravo"));
+TEST(3, true, x.hasOwnProperty("@attr1"));
+TEST(4, false, x.hasOwnProperty("foobar"));
 
-TEST(2, "foo", namespaces[0].prefix);
-TEST(3, "http://foo/", namespaces[0].uri);
-TEST(4, "bar", namespaces[1].prefix);
-TEST(5, "http://bar/", namespaces[1].uri);
-TEST(6, "", namespaces[2].prefix);
-TEST(7, "", namespaces[2].uri);
-TEST(8, 3, namespaces.length);
+// Test for XML Prototype Object - returns true for XML methods.
+TEST(5, true, XML.prototype.hasOwnProperty("toString"));
+TEST(6, false, XML.prototype.hasOwnProperty("foobar"));
 
 END();
\ No newline at end of file
