@@ -36,35 +36,43 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-gTestfile = '15.9.5.21-2.js';
+gTestfile = '15.9.1.13-1.js';
 
 /**
-   File Name:          15.9.5.21.js
-   ECMA Section:       15.9.5.21
-   Description:        Date.prototype.getUTCMilliseconds
+   File Name:          15.9.1.13-1.js
+   ECMA Section:       15.9.1.1 MakeDate(day, time)
+   Description:
 
-   1.  Let t be this time value.
-   2.  If t is NaN, return NaN.
-   3.  Return msFromTime(t).
+   The operator MakeDate calculates a number of milliseconds from its
+   two arguments, which must be ECMAScript number values. This
+   operator functions as follows:
 
-   Author:             christine@netscape.com
-   Date:               12 november 1997
+   1. If day is not finite or time is not finite, return NaN.
+
+   2. Compute day * msPerDay + time.
+
+   3. Return Result(2).
 */
 
-var SECTION = "15.9.5.21";
-var VERSION = "ECMA_1";
-startTest();
-var TITLE   = "Date.prototype.getUTCMilliseconds()";
+new TestCase( SECTION,
+              "MakeDate(Number.POSITIVE_INFINITY, 0)",
+              Number.NaN,
+              MakeDate(Number.POSITIVE_INFINITY, 0));
 
-writeHeaderToLog( SECTION + " "+ TITLE);
+new TestCase( SECTION,
+              "MakeDate(Number.NEGATIVE_INFINITY, 0)",
+              Number.NaN,
+              MakeDate(Number.NEGATIVE_INFINITY, 0));
 
-addTestCase( TIME_0000 );
+new TestCase( SECTION,
+              "MakeDate(0, Number.POSITIVE_INFINITY)",
+              Number.NaN,
+              MakeDate(0, Number.POSITIVE_INFINITY));
+
+new TestCase( SECTION,
+              "MakeDate(0, Number.NEGATIVE_INFINITY)",
+              Number.NaN,
+              MakeDate(0, Number.NEGATIVE_INFINITY));
 
 test();
 
-function addTestCase( t ) {
-  new TestCase( SECTION,
-		"(new Date("+t+")).getUTCMilliseconds()",
-		msFromTime(t),
-		(new Date(t)).getUTCMilliseconds() );
-}
