@@ -16,10 +16,10 @@
  *
  * The Initial Developer of the Original Code is
  * Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2005
+ * Portions created by the Initial Developer are Copyright (C) 2007
  * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): Igor Bukanov
+ * Contributor(s): Jesse Ruderman
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -35,39 +35,33 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-var gTestfile = 'regress-311792-02.js';
+var gTestfile = 'regress-351102-04.js';
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 311792;
-var summary = 'Root Array.prototype methods';
-var actual = 'No Crash';
-var expect = 'No Crash';
+var BUGNUMBER = 351102;
+var summary = 'try/catch-guard/finally GC issues';
+var actual = '';
+var expect = '';
 
-printBugNumber(BUGNUMBER);
-printStatus (summary);
+
+//-----------------------------------------------------------------------------
+test();
+//-----------------------------------------------------------------------------
+
+function test()
+{
+  enterFunc ('test');
+  printBugNumber(BUGNUMBER);
+  printStatus (summary);
  
-var subverted = 0;
-
-function index_getter()
-{
-  delete a[0];
-  gc();
-  for (var i = 0; i != 1 << 14; ++i) {
-    var tmp = new String("test");
-    tmp = null;
+  var f;
+  try
+  {
+    try { @foo } catch([] if gc()) { }
   }
-  return 1;
+  catch(ex)
+  {
+  }
+  reportCompare(expect, actual, summary + ': 4');
+
+  exitFunc ('test');
 }
-
-function index_setter(value)
-{
-  subverted = value;
-}
-
-var a = [ Math.sqrt(2), 0 ];
-a.__defineGetter__(1, index_getter);
-a.__defineSetter__(1, index_setter);
-
-a.reverse();
-printStatus(subverted)
-
-  reportCompare(expect, actual, summary);
