@@ -99,24 +99,25 @@ public:
   }
 #endif
 
-  virtual nsIFrame* HitTest(nsDisplayListBuilder* aBuilder, nsPoint aPt,
-                            HitTestState* aState);
+  virtual void HitTest(nsDisplayListBuilder* aBuilder, const nsRect& aRect,
+                       HitTestState* aState, nsTArray<nsIFrame*> *aOutFrames);
   NS_DISPLAY_DECL_NAME("XULTreeColSplitterTarget")
 };
 
-nsIFrame* 
-nsDisplayXULTreeColSplitterTarget::HitTest(nsDisplayListBuilder* aBuilder,
-                                           nsPoint aPt, HitTestState* aState)
+void
+nsDisplayXULTreeColSplitterTarget::HitTest(nsDisplayListBuilder* aBuilder, const nsRect& aRect,
+                                           HitTestState* aState, nsTArray<nsIFrame*> *aOutFrames)
 {
-  nsPoint pt = aPt - aBuilder->ToReferenceFrame(mFrame);
-  // If we are in either the first 4 pixels or the last 4 pixels, we're going to
+  nsRect rect = aRect - aBuilder->ToReferenceFrame(mFrame);
+  // If we are in either in the first 4 pixels or the last 4 pixels, we're going to
   // do something really strange.  Check for an adjacent splitter.
   PRBool left = PR_FALSE;
   PRBool right = PR_FALSE;
-  if (mFrame->GetSize().width - nsPresContext::CSSPixelsToAppUnits(4) <= pt.x)
+  if (mFrame->GetSize().width - nsPresContext::CSSPixelsToAppUnits(4) <= rect.XMost()) {
     right = PR_TRUE;
-  else if (nsPresContext::CSSPixelsToAppUnits(4) > pt.x)
+  } else if (nsPresContext::CSSPixelsToAppUnits(4) > rect.x) {
     left = PR_TRUE;
+  }
 
   // Swap left and right for RTL trees in order to find the correct splitter
   if (mFrame->GetStyleVisibility()->mDirection == NS_STYLE_DIRECTION_RTL) {
@@ -136,11 +137,10 @@ nsDisplayXULTreeColSplitterTarget::HitTest(nsDisplayListBuilder* aBuilder,
 
     if (child && child->GetContent()->NodeInfo()->Equals(nsGkAtoms::splitter,
                                                          kNameSpaceID_XUL)) {
-      return child;
+      aOutFrames->AppendElement(child);
     }
   }
-  
-  return nsnull;
+
 }
 
 nsresult
