@@ -866,7 +866,19 @@ bool UIFileExists(const string& path)
 
 bool UIMoveFile(const string& file, const string& newfile)
 {
-  return (rename(file.c_str(), newfile.c_str()) != -1);
+  if (!rename(file.c_str(), newfile.c_str()))
+    return true;
+  if (errno != EXDEV)
+    return false;
+
+  NSFileManager *fileManager = [NSFileManager defaultManager];
+  NSString *source = [fileManager stringWithFileSystemRepresentation:file.c_str() length:file.length()];
+  NSString *dest = [fileManager stringWithFileSystemRepresentation:newfile.c_str() length:newfile.length()];
+  if (!source || !dest)
+    return false;
+
+  [fileManager movePath:source toPath:dest handler:nil];
+  return UIFileExists(newfile);
 }
 
 bool UIDeleteFile(const string& file)
